@@ -27,7 +27,7 @@ create_random_image (pixman_format_code_t *allowed_formats,
     uint32_t *buf;
     pixman_image_t *img;
 
-    while (allowed_formats[n] != -1)
+    while (allowed_formats[n] != PIXMAN_null)
 	n++;
     fmt = allowed_formats[lcg_rand_n (n)];
 
@@ -78,7 +78,7 @@ free_random_image (uint32_t initcrc,
     uint32_t *data = pixman_image_get_data (img);
     int height = pixman_image_get_height (img);
 
-    if (fmt != -1)
+    if (fmt != PIXMAN_null)
     {
 	/* mask unused 'x' part */
 	if (PIXMAN_FORMAT_BPP (fmt) - PIXMAN_FORMAT_DEPTH (fmt) &&
@@ -218,7 +218,7 @@ static pixman_format_code_t img_fmt_list[] = {
     PIXMAN_a1r1g1b1,
     PIXMAN_a1b1g1r1,
     PIXMAN_a1,
-    -1
+    PIXMAN_null
 };
 
 static pixman_format_code_t mask_fmt_list[] = {
@@ -226,7 +226,7 @@ static pixman_format_code_t mask_fmt_list[] = {
     PIXMAN_a8,
     PIXMAN_a4,
     PIXMAN_a1,
-    -1
+    PIXMAN_null
 };
 
 
@@ -247,7 +247,7 @@ test_composite (int testnum, int verbose)
     int dst_x, dst_y;
     int mask_x, mask_y;
     int w, h;
-    int op;
+    pixman_op_t op;
     pixman_format_code_t src_fmt, dst_fmt, mask_fmt;
     uint32_t *dstbuf, *srcbuf, *maskbuf;
     uint32_t crc32;
@@ -305,7 +305,7 @@ test_composite (int testnum, int verbose)
     dst_y = lcg_rand_n (dst_height);
 
     mask_img = NULL;
-    mask_fmt = -1;
+    mask_fmt = PIXMAN_null;
     mask_x = 0;
     mask_y = 0;
     maskbuf = NULL;
@@ -385,7 +385,7 @@ test_composite (int testnum, int verbose)
 	printf ("---\n");
     }
 
-    free_random_image (0, src_img, -1);
+    free_random_image (0, src_img, PIXMAN_null);
     crc32 = free_random_image (0, dst_img, dst_fmt);
 
     if (mask_img)
@@ -393,66 +393,13 @@ test_composite (int testnum, int verbose)
 	if (srcbuf == maskbuf)
 	    pixman_image_unref(mask_img);
 	else
-	    free_random_image (0, mask_img, -1);
+	    free_random_image (0, mask_img, PIXMAN_null);
     }
 
     FLOAT_REGS_CORRUPTION_DETECTOR_FINISH ();
     return crc32;
 }
 
-#define CONVERT_15(c, is_rgb)						\
-    (is_rgb?								\
-     ((((c) >> 3) & 0x001f) |						\
-      (((c) >> 6) & 0x03e0) |						\
-      (((c) >> 9) & 0x7c00)) :						\
-     (((((c) >> 16) & 0xff) * 153 +					\
-       (((c) >>  8) & 0xff) * 301 +					\
-       (((c)      ) & 0xff) * 58) >> 2))
-
-static void
-initialize_palette (pixman_indexed_t *palette, uint32_t mask, int is_rgb)
-{
-    int i;
-
-    for (i = 0; i < 32768; ++i)
-	palette->ent[i] = lcg_rand() & mask;
-
-    for (i = 0; i < mask + 1; ++i)
-    {
-	uint32_t rgba24;
- 	pixman_bool_t retry;
-	uint32_t i15;
-
-	/* We filled the rgb->index map with random numbers, but we
-	 * do need the ability to round trip, that is if some indexed
-	 * color expands to an argb24, then the 15 bit version of that
-	 * color must map back to the index. Anything else, we don't
-	 * care about too much.
-	 */
-	do
-	{
-	    uint32_t old_idx;
-	    
-	    rgba24 = lcg_rand();
-	    i15 = CONVERT_15 (rgba24, is_rgb);
-
-	    old_idx = palette->ent[i15];
-	    if (CONVERT_15 (palette->rgba[old_idx], is_rgb) == i15)
-		retry = 1;
-	    else
-		retry = 0;
-	} while (retry);
-	
-	palette->rgba[i] = rgba24;
-	palette->ent[i15] = i;
-    }
-
-    for (i = 0; i < mask + 1; ++i)
-    {
-	assert (palette->ent[CONVERT_15 (palette->rgba[i], is_rgb)] == i);
-    }
-}
-
 int
 main (int argc, const char *argv[])
 {
@@ -460,8 +407,8 @@ main (int argc, const char *argv[])
 
     for (i = 1; i <= 8; i++)
     {
-	initialize_palette (&(rgb_palette[i]), (1 << i) - 1, TRUE);
-	initialize_palette (&(y_palette[i]), (1 << i) - 1, FALSE);
+	initialize_palette (&(rgb_palette[i]), i, TRUE);
+	initialize_palette (&(y_palette[i]), i, FALSE);
     }
 
     return fuzzer_test_main("blitters", 2000000,
